new Autocomplete("address_osm", {
  selectFirst:      true,
  insertToInput:    true,
  cache:            true,
  howManyCharacters: 3,
  
  // onSearch
  onSearch: ({ currentValue }) => {
    // api
    const api = `https://nominatim.openstreetmap.org/search?format=geojson&limit=5&addressdetails=1&q=${encodeURI(
      currentValue
    )}`;

    /**
     * Promise
     */
    return new Promise((resolve) => {
      fetch(api)
        .then((response) => response.json())
        .then((data) => {
          resolve(data.features);
        })
        .catch((error) => {
          console.error(error);
        });
    });
  },

  // nominatim GeoJSON format
  onResults: ({ currentValue, matches, template }) => {
    const regex = new RegExp(currentValue, "gi");

    // if the result returns 0 we
    // show the no results element
    return matches === 0
      ? template
      : matches
          .map((element) => {
            return `
                <li>
                  <p>
                    ${element.properties.display_name.replace(
                      regex,
                      (str) => `<b>${str}</b>`
                    )}
                  </p>
                </li> `;
          })
          .join("");
  },

  onSubmit: ({ object }) => {
    const { display_name, address, name } = object.properties;
    const [lng, lat] = object.geometry.coordinates;

    marker.setLatLng([lat, lng]);
    marker.setPopupContent(display_name);
    marker.setTooltipContent(name);
        
    map.flyTo([lat, lng], 8, {
        duration: 1,
        animate: true,
    });
    
        jQuery("#location_latitude").val(lat);
        jQuery("#location_longitude").val(lng);
        jQuery("#location_address").val(address.street ?? address.road ?? '');
        jQuery("#location_postcode").val(address.postcode ?? '');
        jQuery("#location_city").val(address.city ?? address.town ?? address.village ?? '');
        jQuery("#location_countrycode").val(address.country_code ?? '');

    console.log("onSubmit:", { object });
  },

  // get index and data from li element after
  // hovering over li with the mouse or using
  // arrow keys ↓ | ↑
  onSelectedItem: ({ index, element, object }) => {
    console.log("onSelectedItem:", { index, element, object });
  },

  // the method presents no results
  // no results
  noResults: ({ currentValue, template }) =>
    template(`<li>No results found: "${currentValue}"</li>`),
});

