<?php

/*
 * Plugin Name:       dev/null OpenStreetMap for Bricks
 * Plugin URI:        https://www.dev-null.at/bricks/open-street-maps
 * Description:       OpenStreetMap - Bricks element.
 * Version:           1.3
 * Requires at least: 6.7
 * Requires PHP:      8.0
 * Author:            Wolfgang Rotschek
 * Author URI:        https://www.dev-null.at/
 * License:           GPL v2 or later
 * License URI:       https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain:       dev-null-osm
 * Domain Path:       /languages
 * Requires Plugins:  
 */

define( 'DEV_NULL_LEAFLET_PATH', __DIR__ );    // require_once files

define( 'DEV_NULL_LEAFLET_ICON',		'leaflet_icon');
define( 'DEV_NULL_LEAFLET_LOCATION', 	'leaflet_location');
define( 'DEV_NULL_LEAFLET_TYPE', 		'leaflet_location_type');
define( 'DEV_NULL_LEAFLET_GROUP', 		'leaflet_location_group');

define( 'DEV_NULL_OSM_PLUGIN_URL', 		plugins_url(basename(dirname(__FILE__))));

load_plugin_textdomain( 'dev-null-osm', false, dirname( plugin_basename( __FILE__ ) ) . '/languages' );

require_once DEV_NULL_LEAFLET_PATH . '/includes/init.php';

/**
 * Register custom elements
 */
add_action( 'init', function() {

	$dev_null_leaflet_includes = [
		'/shortcodes/dev_null_leaflet_map.php',
	];

	$dev_null_leaflet_elements = [
		__DIR__ . '/elements/element-leaflet-map.php',
	];

	foreach ( $dev_null_leaflet_includes as $file ) {
		require_once( __DIR__ . $file );
	}

	foreach ( $dev_null_leaflet_elements as $file ) {
		\Bricks\Elements::register_element( $file );
	}

	wp_register_script( 'leaflet',
        DEV_NULL_OSM_PLUGIN_URL . '/assets/leaflet/leaflet.js',
        array(),
        '1.9.4');
	wp_register_style( 'leaflet',
        DEV_NULL_OSM_PLUGIN_URL . '/assets/leaflet/leaflet.css',
        array(),
        '1.9.4');

	wp_register_script( 'brunob-leaflet.fullscreen',
		DEV_NULL_OSM_PLUGIN_URL . '/assets/brunob.leaflet-fullscreen/Control.FullScreen.js',
        array('leaflet'),
        '4.0.0');
	wp_register_style( 'brunob-leaflet.fullscreen',
		plugins_url('/dev-null-osm/assets/brunob.leaflet-fullscreen/Control.FullScreen.css'),
        array('leaflet'),
        '4.0.0');
	
	wp_register_script( 'tomickigrzegorz.autocomplete',
        plugins_url('/dev-null-osm/assets/tomickigrzegorz.autocomplete/dist/js/autocomplete.js'),
        array('leaflet'),
        '2.0.3');
	wp_register_style( 'tomickigrzegorz.autocomplete',
        plugins_url('/dev-null-osm/assets/tomickigrzegorz.autocomplete/dist/css/autocomplete.css'),
        array('leaflet'),
        '2.0.3');

    wp_register_script('dev-null-osm-kml',
        DEV_NULL_OSM_PLUGIN_URL . '/assets/windycom.leaflet-kml/L.KML.js',
        array('leaflet'),
        '1.1.0');
    wp_register_script('dev-null-osm-gpx',
        DEV_NULL_OSM_PLUGIN_URL . '/assets/mpetazzoni.leaflet-gpx/gpx.js',
        array('leaflet'),
        '2.1.2');
	
	wp_register_script( 'dev-null-osm', false, 
		[
			'leaflet',
			'brunob-leaflet.fullscreen',
			'tomickigrzegorz.autocomplete',
			'dev-null-osm-kml',
			'dev-null-osm-gpx'
		]);
	wp_register_style( 'dev-null-osm', false, ['leaflet', 'brunob-leaflet.fullscreen']);
    
}, 11 );

function dev_null_osm_plugin_activate() {

	$iconsBase = DEV_NULL_LEAFLET_PATH . '/assets/leaflet-markers';

	$query = array(
		'post_type'        	=> array( DEV_NULL_LEAFLET_ICON ),
		'posts_per_page'	=> -1,
	);
			
	$icons = get_posts($query);

	$list = [];
	
	foreach($icons as $icon) {
		$list[$icon->post_name] = array(
			'ID'	=> $icon->ID,
			'title' => $icon->post_title,
		);
	}

	$folders = [
		[
			'folder' 			=> '/needles/',
			'icon_size_x'	    => 48,
			'icon_size_y'	    => 48,
			'icon_anchor_x'		=> 24,
			'icon_anchor_y'		=> 46,
			'popup_anchor_x'	=> 1,
			'popup_anchor_y'	=> -34,
		],[
			'folder' => '/pins/',
			'icon_size_x'	    => 48,
			'icon_size_y'	    => 48,
			'icon_anchor_x'		=> 24,
			'icon_anchor_y'		=> 46,
			'popup_anchor_x'	=> 1,
			'popup_anchor_y'	=> -34,
		],[
			'folder' => '/mapsmarker.com/',
			'icon_size_x'	    => 32,
			'icon_size_y'	    => 37,
			'icon_anchor_x'		=> 16,
			'icon_anchor_y'		=> 37,
			'popup_anchor_x'	=> 1,
			'popup_anchor_y'	=> -34,
		],[
			'folder' => '/nchart/',
			'icon_size_x'	    => 41,
			'icon_size_y'	    => 41,
			'icon_anchor_x'		=> 20,
			'icon_anchor_y'		=> 20,
			'popup_anchor_x'	=> 0,
			'popup_anchor_y'	=> -10,
		],
	];
	
	foreach($folders as $folder)
	{
		$files = scandir($iconsBase . $folder['folder']);

		foreach($files as $file) {
			$base = pathinfo($file);
			
			if($base['extension'] != 'png')
				continue;
		
			$base['title'] = ucfirst(trim(str_replace(['-', '_'], ' ', $base['filename'])));
			$postArray = [
				'post_title'   => $base['title'],
				'post_content' => '',
				'post_status'  => 'publish',
				'post_author'  => get_current_user_id(),
				'meta_input'   => [
					'icon_url' 			=> plugins_url('/assets/leaflet-markers', __FILE__ ) . $folder['folder'] . $file,
					'icon_size_x'	    => $folder['icon_size_x'],
					'icon_size_y'	    => $folder['icon_size_y'],
					'icon_anchor_x'		=> $folder['icon_anchor_x'],
					'icon_anchor_y'		=> $folder['icon_anchor_y'],
					'popup_anchor_x'	=> $folder['popup_anchor_x'],
					'popup_anchor_y'	=> $folder['popup_anchor_y'],

				],
				'post_type'		=> DEV_NULL_LEAFLET_ICON,
			];

			$slug = str_replace([' ', '_', '.'], '-', $base['filename']);
			
			if(isset($list[$slug])) {
				$postArray['ID']	= $list[$slug]['ID'];
			} else {
				$postArray['ID']	= 0;
			}
			wp_insert_post($postArray);
		}
	}
	return;
  
}

register_activation_hook( __FILE__, 'dev_null_osm_plugin_activate' );