<?php
namespace DevNullLeaflet;

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

class Element_Leaflet_Map extends \Bricks\Element {
	public $category  = 'general';
	public $name      = 'dev-null-leaflet-map';
	public $icon      = 'ti-location-pin';
	public $scripts   = [ ];
	public $draggable = false;

	private $defaultOptionsGpx = 
"var optionsGpx = {
	async: true, 
	markers: {
//		startIcon: null, 
//		endIcon: null,
		wptIcons: {
		},
	},
};
";


	public function get_label() {
		return esc_html__( 'Open Street Map', 'dev-null-osm' );
	}

	public function enqueue_scripts() {
        
		wp_enqueue_style( 'dev-null-osm');
   		wp_enqueue_script( 'dev-null-osm');
	}

	public function set_control_groups() {
		$this->control_groups['placeholder'] = [
			'title'    => esc_html__( 'Placeholder', 'dev-null-osm' ),
		];
        
		$this->control_groups['deprecated'] = [
			'title'    => esc_html__( 'Deprecated', 'dev-null-osm' ),
		];
        

		$this->control_groups['map_layers'] = [
			'title'    => esc_html__( 'Layer options', 'dev-null-osm' ),
		];
        
		$this->control_groups['settings'] = [
			'title'    => esc_html__( 'Settings', 'dev-null-osm' ),
		];
        
	}

	public function set_controls() {

		// get sorted list of all database items
		$LeafletItems = apply_filters('dev_null_leaflet_items', []);
		uasort($LeafletItems, 'strcasecmp');
		
		$this->controls['LeafletItems'] = [
			'label'       => esc_html__( 'Map items', 'dev-null-osm' ),
			'type'        => 'select',
			'options'     => bricks_is_builder() ? $LeafletItems : [],
			'clearable'		=> true,
			'multiple'		=> true,
			'searchable'  => true,
			'placeholder' => esc_html__( 'Select Map items', 'dev-null-osm' ),
		];

		$this->controls['placeholder_height'] = [
			'group'       => 'placeholder',
			'label'       => esc_html__( 'Placeholder Height', 'dev-null-osm' ),
			'type'        => 'number',
			'units'       => true,
			'placeholder' => '400px',
		];

		$this->controls['placeholder_width'] = [
			'group'       => 'placeholder',
			'label'       => esc_html__( 'Placeholder Width', 'dev-null-osm' ),
			'type'        => 'number',
			'units'       => true,
			'placeholder' => '100%',
		];

		$this->controls['infoDeprecated'] = [
			'group'       => 'deprecated',
			'content'  => esc_html__( 'Theses settings are deprecated and will be removed in one of the next versions. Please move the selected items to Map items', 'dev-null-osm' ),
			'type'     => 'info',
		];

		$this->controls['LocationType'] = [
			'group'       => 'deprecated',
			'label'       => esc_html__( 'Location record type', 'dev-null-osm' ),
			'type'        => 'select',
			'options'     => [
				'item'   => esc_html__( 'Location', 'dev-null-osm' ),
				'group' => esc_html__( 'Group', 'dev-null-osm' ),
			],
			'placeholder' => esc_html__( 'Group', 'dev-null-osm' ),
		];

		$this->controls['LocationItems'] = [
			'group'       => 'deprecated',
			'label'       => esc_html__( 'Locations', 'dev-null-osm' ),
			'type'        => 'select',
			'options'     => bricks_is_builder() ? \DevNullLeaflet\Leaflet_Location_Item::get_LocationItemList() : [],
			'clearable'		=> true,
			'multiple'		=> true,
			'searchable'  => true,
			'placeholder' => esc_html__( 'Select location items', 'dev-null-osm' ),
			'required'    => [ 'LocationType', '=', 'item' ],
		];

		$this->controls['LocationGroups'] = [
			'group'       => 'deprecated',
			'label'       => esc_html__( 'Locations', 'dev-null-osm' ),
			'type'        => 'select',
			'options'     => bricks_is_builder() ? \DevNullLeaflet\Leaflet_Location_Group::get_LocationGroupList() : [],
			'clearable'		=> true,
			'multiple'		=> true,
			'searchable'  => true,
			'placeholder' => esc_html__( 'Select marker group', 'dev-null-osm' ),
			'required'    => [ 'LocationType', '!=', 'item' ],
		];

		$this->controls['infoExecuteCode'] = [
			'group'       => 'map_layers',
			'content'  => esc_html__( 'The executed code will run on your site! Only add code that you consider safe.', 'dev-null-osm' ),
			'type'     => 'info',
		];

		$this->controls['optionsGpx'] = [
			'group'        => 'map_layers',
			'label'        => esc_html__( 'Custom GPX options', 'dev-null-osm' ),
			'type'         => 'code',
			'placeholder'  => $this->defaultOptionsGpx,
			'mode'         => 'javascript',
			'hasVariables' => false,
			// translators: 1: link to git */
			'description'  => sprintf( esc_html__( 'Available GPX options reference at %s', 'dev-null-osm' ), '<a target="_blank" href="https://github.com/mpetazzoni/leaflet-gpx">https://github.com/mpetazzoni/leaflet-gpx</a>' ),
		];

		/**
		 * Group: SETTINGS
		**/

		$this->controls['padding'] = [
			'group'       => 'settings',
			'label'       => esc_html__( 'Padding map bounds', 'dev-null-osm' ),
			'type'        => 'number',
			'step'        => 1,
			'min'         => 0,
			'max'         => 50,
			'placeholder' => 30,
		];

		$this->controls['draggable'] = [
			'group'    => 'settings',
			'label'    => esc_html__( 'Draggable', 'dev-null-osm' ),
			'type'     => 'checkbox',
			'default'  => true,
			'required' => [ 'customOptions', '=', ''],
		];

		$this->controls['zoomControl'] = [
			'group'    => 'settings',
			'label'    => esc_html__( 'Zoom Control', 'dev-null-osm' ),
			'type'     => 'checkbox',
			'default'  => 1,
			'required' => [ 'customOptions', '=', ''],
		];

		$this->controls['zoom'] = [
			'group'       => 'settings',
			'label'       => esc_html__( 'Zoom level', 'dev-null-osm' ),
			'type'        => 'number',
			'step'        => 1,
			'min'         => 0,
			'max'         => 20,
			'placeholder' => 12,
		];

		$this->controls['minZoom'] = [
			'group'    => 'settings',
			'label'    => esc_html__( 'Zoom level', 'dev-null-osm' ) . ' (' . esc_html__( 'Min', 'dev-null-osm' ) . ')',
			'type'     => 'number',
			'step'     => 1,
			'min'      => 1,
			'placeholder' => 1,
			'required' => [
				[ 'zoomControl', '!=', '' ],
				[ 'customOptions', '=', ''],
			],
		];

		$this->controls['maxZoom'] = [
			'group'    => 'settings',
			'label'    => esc_html__( 'Zoom level', 'dev-null-osm' ) . ' (' . esc_html__( 'Max', 'dev-null-osm' ) . ')',
			'type'     => 'number',
			'step'     => 1,
			'min'      => 0,
			'max'      => 20,
			'placeholder' => 20,
			'required' => [
				[ 'zoomControl', '!=', '' ],
				[ 'customOptions', '=', ''],
			],
		];

		$this->controls['fullscreenControl'] = [
			'group'    => 'settings',
			'label'    => esc_html__( 'Fullscreen Control', 'dev-null-osm' ),
			'type'     => 'checkbox',
			'default'  => true,
			'required' => [
				[ 'zoomControl', '!=', '' ],
				[ 'customOptions', '=', ''],
			],
		];

		$this->controls['boxZoom'] = [
			'group'    => 'settings',
			'label'    => esc_html__( 'Box Zoom', 'dev-null-osm' ),
			'type'     => 'checkbox',
			'default'  => true,
			'required' => [
				[ 'zoomControl', '!=', '' ],
				[ 'customOptions', '=', ''],
			],
		];

		$this->controls['scrollwheel'] = [
			'group'    => 'settings',
			'label'    => esc_html__( 'Scroll', 'dev-null-osm' ),
			'type'     => 'checkbox',
			'default'  => true,
			'required' => [
				[ 'zoomControl', '!=', '' ],
				[ 'customOptions', '=', ''],
			],
		];

		// $this->controls['customOptions'] = [
			// 'group'        => 'settings',
			// 'label'        => esc_html__( 'Custom map options', 'dev-null-osm' ),
			// 'type'         => 'code',
			// 'placeholder'  => "zoomControl: true,\nminZoom: 1,\nmaxZoom: 14",
			// 'mode'         => 'txt', // 'javascript',
			// 'hasVariables' => false,
			// /* translators: 1: link to leafletjs */
			// 'description'  => sprintf( esc_html__( 'Available map options reference at %s', 'dev-null-osm' ), '<a target="_blank" href="https://leafletjs.com/reference.html">leafletjs.com/reference.html</a>' ),
		// ];

	}

	public function render() {

		// control defaults
		$defaults = [
			'LeafletItems'		=> [],
            'zoom'      		=> 12,
            'minZoom'   		=> 1,
            'maxZoom'   		=> 20,
            'width'     		=> '100%',
            'height'    		=> '400px',
			'LocationType'		=> 'group',
			'LocationGroups'	=> array(),
			'LocationItems'		=> array(),
            'layer_kml'         => array(),
			'zoomControl'		=> 1,
			'draggable'			=> 1,
			'boxZoom'			=> 1,
			'scrollwheel'		=> 1,
			'padding'			=> 30,
			'customOptions'		=> '',
            'fullscreenControl' => 1,
			'optionsGpx'		=> $this->defaultOptionsGpx,
			'placeholder_height' => '400px',
			'placeholder_width' => '100%',			
        ];
        
		
		// control settings from designer
		$settings = $this->settings;

		if( bricks_is_builder_call() || bricks_is_builder_iframe() || bricks_is_builder_main() )
		{
			return $this->render_element_placeholder(
				[
					'title' => esc_html__( 'Leaflet Map', 'dev-null-osm' ) . ': ' . esc_html__( 'Don\'t render in builder', 'dev-null-osm' ),
					'style' => [
						'width'  => isset($settings['placeholder_width']) ? esc_attr($settings['placeholder_width']) : '100%',
						'height' => isset($settings['placeholder_height']) ? esc_attr($settings['placeholder_height']) : '400px',
					],
				]
			);
		}

		$leafletItems = [
			'pnt'	=> [],
			'grp'	=> [],
			'kml'	=> [],
			'gpx'	=> [],
			'ctr'	=> [],
		];

		foreach($settings['LeafletItems'] as $key)
		{
			$itm = explode(':', $key);
			
			if(isset($leafletItems[$itm[0]]) == false)
				$leafletItems[$itm[0]] = [];
			
			$leafletItems[$itm[0]][] = $itm[1];
		}

        $atts = [
			'LeafletItems'		=> isset($settings['LeafletItems']) ? $settings['LeafletItems'] : [], 
            'zoom'      		=> isset($settings['zoom']) ? intval($settings['zoom']) : 12,
            'minZoom'   		=> isset($settings['minZoom']) && is_numeric($settings['minZoom'])
										? intval($settings['minZoom']) : 1,
            'maxZoom'   		=> isset($settings['maxZoom']) && is_numeric($settings['maxZoom'])
										? intval($settings['maxZoom']) : 20,
            'width'     		=> isset($settings['width']) && $settings['width'] <> '' ? esc_attr($settings['width']) : '100%',
            'height'    		=> isset($settings['height']) && $settings['height'] <> '' ? esc_attr($settings['height']) : '400px',
			'LocationType'		=> isset($settings['LocationType']) && $settings['LocationType'] <> '' 
										? esc_attr($settings['LocationType']) : 'group',
			'LocationGroups'	=> isset($settings['LocationGroups']) 
										? array_map('intval', $settings['LocationGroups']) : [],
			'LocationItems'		=> isset($settings['LocationItems']) 
										? array_map('intval', $settings['LocationItems']) : [],
            'layer_kml'         => isset($leafletItems['kml']) 
										? array_map('intval', $leafletItems['kml']) : [],
            'layer_gpx'         => isset($leafletItems['gpx']) 
										? array_map('intval', $leafletItems['gpx']) : [],
			'zoomControl'		=> isset($settings['zoomControl']) && intval($settings['zoomControl']) == 1 ? 'true' : 'false',
			'draggable'			=> isset($settings['draggable']) && intval($settings['draggable']) == 1 ? 'true' : 'false',
			'boxZoom'			=> isset($settings['boxZoom']) && intval($settings['boxZoom']) == 1 ? 'true' : 'false',
			'scrollwheel'		=> isset($settings['scrollwheel']) && intval($settings['scrollwheel']) == 1 ? 'true' : 'false',
			'padding'			=> isset($settings['padding']) ? intval($settings['padding']) : 30,
			'customOptions'		=> '',
            'fullscreenControl' => isset($settings['fullscreenControl']) && intval($settings['fullscreenControl']) == 1 ? 'true' : 'false',
			'optionsGpx'		=> $this->defaultOptionsGpx,
			'placeholder_height' => isset($settings['placeholder_height']) && $settings['placeholder_height'] <> '' 
										? esc_attr($settings['placeholder_height']) : '400px',
			'placeholder_width' => isset($settings['placeholder_height']) && $settings['placeholder_height'] <> '' 
										? esc_attr($settings['placeholder_height']) : '100%',			
		];

		// depreciated - will be removed
		if(isset($settings['LocationItems']) && $atts['LocationType'] = 'location')
			$leafletItems['pnt'] = array_merge($leafletItems['pnt'], $settings['LocationItems']);
		else if(isset($settings['LocationGroups']) && $atts['LocationType'] = 'group')
			$leafletItems['grp'] = array_merge($leafletItems['grp'], $settings['LocationGroups']);

        $optZoomControl		 		= $atts['zoomControl'];
        $optFullscreenControl		= $atts['fullscreenControl'];
        $optBoxZoom					= $atts['boxZoom'];
        $optScrollWheel				= $atts['scrollwheel'];

        $atts['minZoom'] 			= (is_numeric($atts['minZoom']) ? $atts['minZoom'] : $defaults['minZoom']);
        $atts['maxZoom'] 			= (is_numeric($atts['maxZoom']) ? $atts['maxZoom'] : $defaults['maxZoom']);
        $atts['zoom'] 				= (is_numeric($atts['zoom']) ? $atts['zoom'] : $defaults['zoom']);
		$optDraggable	 			= $atts['draggable'];
		$optPadding 				= (is_numeric($atts['padding']) ? $atts['padding'] : $defaults['padding']);

		// STEP: Get JavaScript code
		if ( $atts['customOptions'] ) {
			// Regular expression to detect escaping script tag, including those with attributes
			$script_tag_pattern = '/<\/script\b[^>]*>/mi';

			// Check the code if it tries to escape with a </script>
			if ( preg_match( $script_tag_pattern, $atts['customOptions'] ) ) {
				$atts['customOptions'] = "/* invalid </script> tag detected */";
			}
		}

		$locations = [];
		
		$locations = \DevNullLeaflet\Leaflet_Location_Item::load_items($locations, $leafletItems['pnt']);		
		$locations = \DevNullLeaflet\Leaflet_Location_Group::load_items($locations, $leafletItems['grp']);
		$locations = \DevNullLeaflet\Leaflet_Location_Country::load_items($locations, $leafletItems['ctr']);
		
		$mapParts = array();
		$mapParts = \DevNullLeaflet\Leaflet_Map::get_LeafletMapParts($locations, $atts);

		$mapID = 'osm_map_'.$this->element['id'];
		// $this->attributes['_root']['id'];

		$script_opt = $atts['customOptions'];
		
		if($script_opt == '') {
			$script_opt = "
const options = {
	zoomDelta:          0.5,
	zoomSnap:           0.5,
	zoomControl:        {$optZoomControl},
	minZoom:            {$atts['minZoom']},
	maxZoom:            {$atts['maxZoom']},
	dragging:           {$optDraggable},
	doubleClickZoom:    {$optZoomControl},
	boxZoom:            {$optBoxZoom},
	scrollWheelZoom:    {$optScrollWheel},
	fullscreenControl:  {$optFullscreenControl},
	fullscreenControlOptions: {
		position:           'topright',
		forceSeparateButton: true,
	},
};
";
		}
		
		$script_map = implode("", $mapParts['init']);
		$script_map .= $script_opt;
		$script_map .= "

const map = L.map('{$mapID}', options );
";

		$script_map .= $mapParts['tiles'];
		$script_map .= implode("", $mapParts['icons']);
		$script_map .= implode("", $mapParts['layers']);
		$script_map .= implode("", $mapParts['mapBounds']);

		wp_add_inline_script('dev-null-osm', $script_map);

		$mapWidth = isset($this->settings['_width']) ? $this->settings['_width'] : 
			(isset($this->settings['placeholder_width']) ? $this->settings['placeholder_width'] : $defaults['width']);
		$mapHeight = isset($this->settings['_height']) ? $this->settings['_height'] : 
			(isset($this->settings['placeholder_height']) ? $this->settings['placeholder_height'] : $defaults['height']);

?>
<div <?php echo $this->render_attributes( '_root' ) . " " . 
	"style='width:" . esc_attr($mapWidth) . "; height: " . esc_attr($mapHeight) . "'" ?>
>
  <div id='<?php echo esc_attr($mapID); ?>' style='width: 100%; height: 100%'>
    #### map content goes here ####
  </div>
</div>
<?php

	}
}
