<?php

namespace DevNullLeaflet;

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

class Leaflet_Location_Group {

	public function __construct() {

		add_action( 'init', [ $this, 'register_taxonomy' ] );
		
	}
	
	public static function load_terms($items, $terms)
	{

		if(is_array($terms) == false)
			$terms = explode(',', $terms);

		if(count($terms) == 0)
			return $items;
		
		$query = [
			'post_type'	=> DEV_NULL_LEAFLET_LOCATION,
			'nopaging'	=> true,
			'tax_query' => [
				[
					'taxonomy' => 'leaflet_location_group',
					'terms' => $terms,
					'field' => 'slug',
				],
			],
		];
		
		$posts = get_posts($query);
		
		foreach($posts as $p) {
			$items[$p->ID] = $p;
		}
		
		return $items;
	}
	
	public static function load_items($items, $list)
	{

		if(is_array($list) == false)
			$list = explode(',', $list);

		$list = array_map('intval', $list);
		
		if(count($list) == 0)
			return $items;
		
		$query = [
			'post_type'	=> DEV_NULL_LEAFLET_LOCATION,
			'nopaging'	=> true,
			'tax_query' => [
				[
					'taxonomy' => 'leaflet_location_group',
					'terms' => $list,
					'field' => 'term_id',
				],
			],
		];
		
		$posts = get_posts($query);
		
		foreach($posts as $p) {
			$items[$p->ID] = $p;
		}
		
		return $items;
	}
	
	public static function tag_row_actions( $actions, $tag ) {
		// Remove 'Quick Edit'
		// unset( $actions['inline hide-if-no-js'] );
		unset( $actions['view'] );

		return $actions;
	}

	function register_taxonomy() {
		$labels = [
			'name'                       => esc_html__( 'Locationgroup', 'dev-null-osm' ),
			'singular_name'              => esc_html__( 'Locationgroup', 'dev-null-osm' ),
			'menu_name'                  => esc_html__( 'Locationgroups', 'dev-null-osm' ),
			'search_items'               => esc_html__( 'Search Locationgroup', 'dev-null-osm' ),
			'popular_items'              => esc_html__( 'Popular Locationgroups', 'dev-null-osm' ),
			'all_items'                  => esc_html__( 'All Locationgroups', 'dev-null-osm' ),
			'parent_item'                => esc_html__( 'Parent Locationgroup', 'dev-null-osm' ),
			'parent_item_colon'          => esc_html__( 'Parent Locationgroup:', 'dev-null-osm' ),
			'edit_item'                  => esc_html__( 'Edit Locationgroup', 'dev-null-osm' ),
			'view_item'                  => esc_html__( 'View Locationgroup', 'dev-null-osm' ),
			'update_item'                => esc_html__( 'Update Locationgroup', 'dev-null-osm' ),
			'add_new_item'               => esc_html__( 'Add New Locationgroup', 'dev-null-osm' ),
			'new_item_name'              => esc_html__( 'New Locationgroup Name', 'dev-null-osm' ),
			'separate_items_with_commas' => esc_html__( 'Separate Locationgroup with commas', 'dev-null-osm' ),
			'add_or_remove_items'        => esc_html__( 'Add or remove Locationgroup', 'dev-null-osm' ),
			'choose_from_most_used'      => esc_html__( 'Choose most used Locationgroups', 'dev-null-osm' ),
			'not_found'                  => esc_html__( 'No Locationgroup found.', 'dev-null-osm' ),
			'no_terms'                   => esc_html__( 'No Locationgroup', 'dev-null-osm' ),
			'filter_by_item'             => esc_html__( 'Filter by Locationgroup', 'dev-null-osm' ),
			'items_list_navigation'      => esc_html__( 'Locationgroup list pagination', 'dev-null-osm' ),
			'items_list'                 => esc_html__( 'Locationgroup list', 'dev-null-osm' ),
			'most_used'                  => esc_html__( 'Most Used', 'dev-null-osm' ),
			'back_to_items'              => esc_html__( '&larr; Go to Locationgroups', 'dev-null-osm' ),
			'text_domain'                => esc_html__( 'dev-null-osm', 'dev-null-osm' ),
		];
		$args = [
			'label'              => esc_html__( 'Locationgroups', 'dev-null-osm' ),
			'labels'             => $labels,
			'description'        => '',
			'public'             => true,
			'publicly_queryable' => true,
			'hierarchical'       => false,
			'show_ui'            => true,
			'show_in_menu'       => true,
			'show_in_nav_menus'  => false,
			'show_in_rest'       => true,
			'show_tagcloud'      => false,
			'show_in_quick_edit' => true,
			'show_admin_column'  => true,
			'query_var'          => true,
			'sort'               => false,
			'_slug_changed'      => true,
			'meta_box_cb'        => 'post_tags_meta_box',
			'rest_base'          => '',
			'rewrite'            => [
				'with_front'   => false,
				'hierarchical' => false,
			],
		];
		register_taxonomy( DEV_NULL_LEAFLET_GROUP, [DEV_NULL_LEAFLET_LOCATION], $args );
	}

	public static function get_LocationGroupList() {
		$terms = get_terms(
					array(
						'taxonomy'   => DEV_NULL_LEAFLET_GROUP,
						'hide_empty' => false,
					) );
		$list = array();

		foreach($terms as $term) {
			$list[$term->term_id] = esc_attr($term->name) . ' (' . $term->count . ')';
		}
		return $list;
	}
	
    public static function item_list_groups($items) {
        
		$terms = get_terms(
					array(
						'taxonomy'   => DEV_NULL_LEAFLET_GROUP,
						'hide_empty' => false,
					) );
		foreach($terms as $term) {
			$items['grp:' . $term->term_id] = esc_attr($term->name) . ' [grp]';
		}

		return $items;
        
    }
    

		
}

add_filter('dev_null_leaflet_items', 
	[ '\DevNullLeaflet\Leaflet_Location_Group', 'item_list_groups'], 10, 1);

add_filter( DEV_NULL_LEAFLET_GROUP . '_row_actions',
	[ '\DevNullLeaflet\Leaflet_Location_Group', 'tag_row_actions' ], 10, 2 );
