<?php

namespace DevNullLeaflet;

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly


class Leaflet_Map {
	
	public static function get_LeafletMapParts($locations, $atts = []) {
		
		$mapParts = array(
			'init'		=> [],
			'icons'		=> [],
			'layers'	=> [],
			'mapBounds'	=> [],
			'tiles'		=> "",
			);
			
//** map part INIT

		$mapParts['init'][] = "
	var icons = [];
	var bounds = null;
";

//** map part TILES

		$mapParts['tiles'] = "
	const tiles = L.tileLayer('https://tile.openstreetmap.org/{z}/{x}/{y}.png', {
		attribution: '&copy; <a href=\"http://www.openstreetmap.org/copyright\">OpenStreetMap</a>'
	}).addTo(map);
";

		if(count($locations) > 0) {
			$mapParts['init'][] = "
	var mapLatLng = [];
";

			foreach($locations as $loc)
			{
				$nam = esc_attr(get_the_title($loc));

				$lat    = esc_js(get_post_meta($loc->ID, 'location_latitude', true));
				$lng    = esc_js(get_post_meta($loc->ID, 'location_longitude', true));
				$icon   = esc_js(get_post_meta($loc->ID, 'location_marker_color', true));
				
				$txt    = get_post_meta($loc->ID, 'location_popuptext', true);
				if($txt == "")
					$txt = $nam;
				else
					$txt = $txt;
				
				$txt	= str_replace(["%title%", "%lat%", "%lng%"], [$nam, $lat, $lng], $txt);
				$txt	= str_replace(["\"", "'", "\n", "\r"], ["\\\"", "&#039;", "\\n", "\\r"], $txt);
				
				$icon = empty($icon) ? '.leaflet' : $icon;

				$mapParts['layers'][] = "
		L.marker(
			[{$lat}, {$lng}],
			{
				title: '{$nam}',
				icon: icons['{$icon}'],
			}
		).bindPopup(\"{$txt}\").addTo(map);
		mapLatLng.push(L.latLng({$lat}, {$lng}));
";

				$mapParts['icons'][$icon] = \DevNullLeaflet\Leaflet_Icon::get_IconScript($icon);
			}

			if(count($locations) == 1) {
				$mapParts['layers'][] = "
		map.setView(mapLatLng[0], {$atts['zoom']});
		bounds = map.getBounds();
";
			} else if(count($locations) > 1) {
				$mapParts['layers'][] = "
		bounds = L.latLngBounds(mapLatLng);
		map.fitBounds(bounds, {padding: [30, 30]});
";
			}
				
		}
		

// ** map part KML

        if(isset($atts['layer_kml']) && (count($atts['layer_kml']) > 0))
		{
            foreach($atts['layer_kml'] as $ndx) {
                $url                    = wp_get_attachment_url( $ndx );
				$pt						= get_post_mime_type( $ndx );
				
				$script = "
	// Load kml file
	fetch('{$url}')
		.then(res => res.text())
		.then(kmltext => {
			// Create new kml overlay
			const parser = new DOMParser();
			const kml = parser.parseFromString(kmltext, 'text/xml');
			const track = new L.KML(kml);
			map.addLayer(track);

			// Adjust map to show the kml
			if(bounds === null)
				bounds = track.getBounds();
			else
				bounds.extend(track.getBounds());
			
			map.fitBounds(bounds, {padding: [25, 25]});
		});
";
				$mapParts["layers"][] = $script;
			}
		}

		if(isset($atts['layer_gpx']) && (count($atts['layer_gpx']) > 0))
		{
			$mapParts['init']['gpx'] = $atts['optionsGpx'];
			
			foreach($atts['layer_gpx'] as $ndx) {
                $url                    = wp_get_attachment_url( $ndx );
						$script = "
		new L.GPX('{$url}', optionsGpx
			).on('addpoint', function(e) {
				console.log('Added ' + e.point_type + ' point: ' + e.point);
			}).on('loaded', (e) => {
			// Adjust map to show the kml
			if(bounds === null)
				bounds = e.target.getBounds();
			else
				bounds.extend(e.target.getBounds());
			
			map.fitBounds(bounds, {padding: [25, 25]});

      }).addTo(map);
";
				$mapParts["layers"][] = $script;
			}
        }

		if(count($mapParts['layers']) == 0) {
			$mapParts['mapBounds'][] = "map.setView([48.2062771,16.3650601], {$atts['zoom']});";
		}

		return $mapParts;		
	}
}