<?php

namespace DevNullLeaflet;

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

add_shortcode( 'dev_null_leaflet_map', 			'DevNullLeaflet\dev_null_leaflet_map');

/* depreciated shortcodes - will be removed */
add_shortcode( 'dev_null_leaflet_location_map', 'DevNullLeaflet\dev_null_leaflet_map');
add_shortcode( 'dev_null_leaflet_country_map', 	'DevNullLeaflet\dev_null_leaflet_map');

function dev_null_leaflet_map($atts) {

    wp_enqueue_style( 'dev-null-osm' );
    wp_enqueue_script( 'dev-null-osm' );

	$defaults = [
		'group'				=> '',
		'id'				=> '',
		'zoom'      		=> 12,
		'minZoom'   		=> 1,
		'maxZoom'   		=> 20,
		'width'     		=> '100%',
		'height'    		=> '400px',
		'zoomControl'		=> 1,
		'draggable'			=> 1,
		'boxZoom'			=> 1,
		'scrollwheel'		=> 1,
		'padding'			=> 30,
		'customOptions'		=> '',
        'fullscreenControl' => 1,
		'country'			=> '',
		'term'				=> '',
	];
	
	$atts = shortcode_atts($defaults, $atts);
	
	$optZoomControl		 		= ($atts['zoomControl'] == 1)										? 'true' : 'false';
	$optFullscreenControl		= ($atts['zoomControl'] == 1 && $atts['fullscreenControl'] == 1)	? 'true' : 'false';
	$optBoxZoom					= ($atts['zoomControl'] == 1 && $atts['boxZoom'] == 1)				? 'true' : 'false';
	$optScrollWheel				= ($atts['zoomControl'] == 1 && $atts['scrollwheel'] == 1)			? 'true' : 'false';

	$atts['minZoom'] 			= (is_numeric($atts['minZoom']) ? $atts['minZoom'] : $defaults['minZoom']);
	$atts['maxZoom'] 			= (is_numeric($atts['maxZoom']) ? $atts['maxZoom'] : $defaults['maxZoom']);
	$atts['zoom'] 				= (is_numeric($atts['zoom']) ? $atts['zoom'] : $defaults['zoom']);
	$optDraggable	 			= ($atts['draggable'] == 1 	? 'true' : 'false');
	$optPadding 				= isset($atts['padding']) 	? intval($atts['padding']) 			: $defaults['padding'];
	$mapCountry					= isset($atts['country']) && $atts['country'] != ''
									? trim(esc_attr($atts['country']))	: '';
	$mapGroups					= isset($atts['grp'])  && $atts['country'] != ''
									? trim(esc_attr($atts['term']))		: '';
	$atts['term']				= isset($atts['term']) && $atts['term'] != ''
									? trim(esc_attr($atts['term']))	: '';
	
	$query = [
		'post_type'         => DEV_NULL_LEAFLET_LOCATION,
		'post_status'       => array( 'publish' ),
		'nopaging'			=> true,
	];

	
	$leafletItems = [
		'pnt'	=> isset($atts['id']) ? explode(',', $atts['id']) : [],
		'grp'	=> isset($atts['grp']) ? explode(",", $atts['grp']) : [],
		'term'	=> isset($atts['term']) ? explode(",", $atts['term']) : [],
		'kml'	=> [],
		'gpx'	=> [],
		'ctr'	=> isset($atts['country']) ? explode(',', $atts['country']) : [],
	];

	$locations = [];
	
// return '<pre>' . print_r($leafletItems, true) . '</pre>';

	$locations = \DevNullLeaflet\Leaflet_Location_Item::load_items($locations, $leafletItems['pnt']);		
	$locations = \DevNullLeaflet\Leaflet_Location_Group::load_terms($locations, $leafletItems['term']);
	$locations = \DevNullLeaflet\Leaflet_Location_Country::load_items($locations, $leafletItems['ctr']);
	
	$height = esc_attr($atts['height']);

	$mapParts = array();
	$mapParts = \DevNullLeaflet\Leaflet_Map::get_LeafletMapParts($locations, $atts);

//	return '<pre>1 ' . print_r($mapParts, true) . '</pre>';
	
	$mapID = 'osm-map';

	$script_opt = $atts['customOptions'];

	if($script_opt == '') {
		$script_opt = "
const options = {
	zoomDelta:          0.5,
	zoomSnap:           0.5,
	zoomControl:        {$optZoomControl},
	minZoom:            {$atts['minZoom']},
	maxZoom:            {$atts['maxZoom']},
	dragging:           {$optDraggable},
	doubleClickZoom:    {$optZoomControl},
	boxZoom:            {$optBoxZoom},
	scrollWheelZoom:    {$optScrollWheel},
	fullscreenControl:  {$optFullscreenControl},
	fullscreenControlOptions: {
		position:           'topright',
		forceSeparateButton: true
	,
	},
};
";
		}
/*
		$script_map = "
var map = L.map('{$mapID}',{
	{$script_opt}
			});

const tiles = L.tileLayer('https://tile.openstreetmap.org/{z}/{x}/{y}.png', {
	maxZoom: {$atts['maxZoom']},
	attribution: '&copy; <a href=\"http://www.openstreetmap.org/copyright\">OpenStreetMap</a>'
}).addTo(map);

const icons = [];

";

	$script_map .= $mapParts['mapBounds'];
	$script_map .= implode("", $mapParts['icons']);
	$script_map .= implode("", $mapParts['marker']);

	wp_add_inline_script('dev-null-osm', $script_map);
*/

		$script_map = implode("", $mapParts['init']);
		$script_map .= $script_opt;
		$script_map .= "

const map = L.map('{$mapID}', options );
";

		$script_map .= $mapParts['tiles'];
		$script_map .= implode("", $mapParts['icons']);
		$script_map .= implode("", $mapParts['layers']);
		$script_map .= implode("", $mapParts['mapBounds']);

		wp_add_inline_script('dev-null-osm', $script_map);
  
    return "<div id='osm-map' style='width: 100%; height: {$height}'></div>";
}